@@ -12,15 +12,19 @@ use warnings;
 
 package Dist::Zilla::Plugin::Test::Compile;
 {
-  $Dist::Zilla::Plugin::Test::Compile::VERSION = '2.017';
+  $Dist::Zilla::Plugin::Test::Compile::VERSION = '2.019';
 }
 # ABSTRACT: common tests to check syntax of your modules
 
 use Moose;
 use Path::Tiny;
-use Data::Section -setup;
+use Sub::Exporter::ForMethods 'method_installer'; # method_installer returns a sub.
+use Data::Section 0.004 # fixed header_re
+    { installer => method_installer }, '-setup';
+
 with (
     'Dist::Zilla::Role::FileGatherer',
+    'Dist::Zilla::Role::FileMunger',
     'Dist::Zilla::Role::TextTemplate',
     'Dist::Zilla::Role::FileFinderUser' => {
         method          => 'found_module_files',
@@ -36,6 +40,7 @@ with (
 );
 
 use Moose::Util::TypeConstraints;
+use namespace::autoclean;
 
 # -- attributes
 
@@ -99,16 +104,34 @@ sub register_prereqs
             phase => 'test',
         },
         'Test::More' => $self->_test_more_version,
-        'Capture::Tiny' => '0',
-        'blib' => '0',
+        'IPC::Open3' => 0,
+        'IO::Handle' => 0,
+        'File::Spec' => 0,
         $self->fake_home ? ( 'File::Temp' => '0' ) : (),
         $self->_script_filenames ? ( 'Test::Script' => '1.05' ) : (),
     );
 }
 
-sub gather_files {
+sub gather_files
+{
+    my $self = shift;
 
-    my ( $self , ) = @_;
+    require Dist::Zilla::File::InMemory;
+
+    for my $file (qw( t/00-compile.t )){
+        $self->add_file( Dist::Zilla::File::InMemory->new(
+            name => $file,
+            content => ${$self->section_data($file)},
+        ));
+    }
+}
+
+sub munge_file
+{
+    my ($self, $file) = @_;
+
+    # cannot check full name, as the file may have been moved by [ExtraTests].
+    return unless $file->name =~ /\b00-compile.t$/;
 
     my @skips = map {; qr/$_/ } $self->skips;
 
@@ -125,39 +148,41 @@ sub gather_files {
     # pod never returns true when loaded
     @module_filenames = grep { !/\.pod$/ } @module_filenames;
 
-    require Dist::Zilla::File::InMemory;
+    my @script_filenames = $self->_script_filenames;
 
-    for my $file (qw( t/00-compile.t )){
-        $self->add_file( Dist::Zilla::File::InMemory->new(
-            name => $file,
-            content => $self->fill_in_string(
-                ${$self->section_data($file)},
-                {
-                    plugin_version => \($self->VERSION),
-                    test_more_version => \($self->_test_more_version),
-                    module_filenames => \@module_filenames,
-                    script_filenames => [ $self->_script_filenames ],
-                    fake_home => \($self->fake_home),
-                    needs_display => \($self->needs_display),
-                    bail_out_on_fail => \($self->bail_out_on_fail),
-                    fail_on_warning => \($self->fail_on_warning),
-                }
-            ),
-        ));
-    }
+    $self->log_debug('adding module ' . $_) foreach @module_filenames;
+    $self->log_debug('adding script ' . $_) foreach @script_filenames;
+
+    $file->content(
+        $self->fill_in_string(
+            $file->content,
+            {
+                dist => \($self->zilla),
+                plugin => \$self,
+                test_more_version => \($self->_test_more_version),
+                module_filenames => \@module_filenames,
+                script_filenames => \@script_filenames,
+                fake_home => \($self->fake_home),
+                needs_display => \($self->needs_display),
+                bail_out_on_fail => \($self->bail_out_on_fail),
+                fail_on_warning => \($self->fail_on_warning),
+            }
+        )
+    );
+
+    return;
 }
 
-no Moose;
 __PACKAGE__->meta->make_immutable;
-1;
 
 =pod
 
 =encoding utf-8
 
-=for :stopwords Jerome Quelin Ahmad Luehrs Karen Etheridge Kent Fredric Marcel Gruenauer
-Olivier Mengu Peter M. Shangov Randy Stauner Ricardo SIGNES fayland Zawawi
-Chris Weyl Harley Pig Jesse
+=for :stopwords Jerome Quelin Ahmad Jesse Luehrs Karen Etheridge Kent Fredric Marcel
+Gruenauer Olivier M. Mengu Peter Shangov Randy Stauner Ricardo SIGNES
+fayland Zawawi Chris Weyl Graham Knop Harley Pig cpantesters FileFinder
+executables AnnoCPAN
 
 =head1 NAME
 
@@ -165,11 +190,11 @@ Dist::Zilla::Plugin::Test::Compile - common tests to check syntax of your module
 
 =head1 VERSION
 
-version 2.017
+version 2.019
 
 =head1 SYNOPSIS
 
-In your dist.ini:
+In your F<dist.ini>:
 
     [Test::Compile]
     skip      = Test$
@@ -203,8 +228,14 @@ This plugin accepts the following options:
 match is done against the module name (C<Foo::Bar>), not the file path
 (F<lib/Foo/Bar.pm>).  This option can be repeated to specify multiple regexes.
 
+=for Pod::Coverage::TrustPod mvp_multivalue_args
+    mvp_aliases
+    register_prereqs
+    gather_files
+    munge_file
+
 =item * fake_home: a boolean to indicate whether to fake C<< $ENV{HOME} >>.
-This may be needed if your module unilateraly creates stuff in homedir:
+This may be needed if your module unilaterally creates stuff in the user's home directory:
 indeed, some cpantesters will smoke test your dist with a read-only home
 directory. Default to false.
 
@@ -225,6 +256,8 @@ warnings during compilation checks. Possible values are:
 installation of modules when upstream dependencies exhibit warnings in a new
 Perl release)
 
+=back
+
 =item * module_finder
 
 This is the name of a L<FileFinder|Dist::Zilla::Role::FileFinder> for finding
@@ -242,18 +275,11 @@ Just like C<module_finder>, but for finding scripts.  The default value is
 C<:ExecFiles> (you can use the L<Dist::Zilla::Plugin::ExecDir> plugin to mark
 those files as executables).
 
-=back
-
 =item * bail_out_on_fail: a boolean to indicate whether the test will BAIL_OUT
 of all subsequent tests when compilation failures are encountered. Defaults to false.
 
 =back
 
-=for Pod::Coverage::TrustPod mvp_multivalue_args
-    mvp_aliases
-    register_prereqs
-    gather_files
-
 =head1 SEE ALSO
 
 L<Test::NeedsDisplay>
@@ -305,6 +331,10 @@ Chris Weyl <cweyl@alumni.drew.edu>
 
 =item *
 
+Graham Knop <haarg@haarg.org>
+
+=item *
+
 Harley Pig <harleypig@gmail.com>
 
 =item *
@@ -356,7 +386,7 @@ ___[ t/00-compile.t ]___
 use strict;
 use warnings;
 
-# This test was generated via Dist::Zilla::Plugin::Test::Compile {{ $plugin_version }}
+# this test was generated with {{ ref($plugin) . ' ' . $plugin->VERSION }}
 
 use Test::More {{ $test_more_version }};
 
@@ -374,14 +404,12 @@ CODE
     : ''
 }}
 
-use Capture::Tiny qw{ capture };
-
-my @module_files = qw(
-{{ join("\n", sort @module_filenames) }}
+my @module_files = (
+{{ join(",\n", map { "    '" . $_ . "'" } map { s/'/\\'/g; $_ } sort @module_filenames) }}
 );
 
-my @scripts = qw(
-{{ join("\n", sort @script_filenames) }}
+my @scripts = (
+{{ join(",\n", map { "    '" . $_ . "'" } map { s/'/\\'/g; $_ } sort @script_filenames) }}
 );
 
 {{
@@ -394,26 +422,27 @@ CODE
     : '# no fake home requested';
 }}
 
+use IPC::Open3;
+use IO::Handle;
+use File::Spec;
+
 my @warnings;
 for my $lib (@module_files)
 {
-    my ($stdout, $stderr, $exit) = capture {
-        system($^X, '-Mblib', '-e', qq{require q[$lib]});
-    };
-    is($?, 0, "$lib loaded ok");
-    warn $stderr if $stderr;
-    push @warnings, $stderr if $stderr;
-}
+    open my $stdout, '>', File::Spec->devnull or die $!;
+    open my $stdin, '<', File::Spec->devnull or die $!;
+    my $stderr = IO::Handle->new;
 
-{{
-($fail_on_warning ne 'none'
-    ? q{is(scalar(@warnings), 0, 'no warnings found')}
-    : '# no warning checks')
-.
-($fail_on_warning eq 'author'
-    ? ' if $ENV{AUTHOR_TESTING};'
-    : ';')
-}}
+    my $pid = open3($stdin, $stdout, $stderr, qq{$^X -Mblib -e"require q[$lib]"});
+    waitpid($pid, 0);
+    is($? >> 8, 0, "$lib loaded ok");
+
+    if (my @_warnings = <$stderr>)
+    {
+        warn @_warnings;
+        push @warnings, @_warnings;
+    }
+}
 
 {{
 @script_filenames
@@ -426,6 +455,16 @@ CODE
     : '';
 }}
 
+{{
+($fail_on_warning ne 'none'
+    ? q{is(scalar(@warnings), 0, 'no warnings found')}
+    : '# no warning checks')
+.
+($fail_on_warning eq 'author'
+    ? ' if $ENV{AUTHOR_TESTING};'
+    : ';')
+}}
+
 {{
 $bail_out_on_fail
     ? 'BAIL_OUT("Compilation problems") if !Test::More->builder->is_passing;'
